# Copyright (c) 2015, Crisco Technologies Pvt. Ltd. and Contributors
# License: MIT. See LICENSE

import json
import os

import criscostack
from criscostack.model.document import Document
from criscostack.modules.export_file import delete_folder


class ModuleDef(Document):
	# begin: auto-generated types
	# This code is auto-generated. Do not modify anything in this block.

	from typing import TYPE_CHECKING

	if TYPE_CHECKING:
		from criscostack.types import DF

		app_name: DF.Literal
		custom: DF.Check
		module_name: DF.Data
		package: DF.Link | None
		restrict_to_domain: DF.Link | None

	# end: auto-generated types
	def on_update(self):
		"""If in `developer_mode`, create folder for module and
		add in `modules.txt` of app if missing."""
		criscostack.clear_cache()
		if not self.custom and criscostack.conf.get("developer_mode"):
			self.create_modules_folder()
			self.add_to_modules_txt()

	def create_modules_folder(self):
		"""Creates a folder `[app]/[module]` and adds `__init__.py`"""
		module_path = criscostack.get_app_path(self.app_name, self.name)
		if not os.path.exists(module_path):
			os.mkdir(module_path)
			with open(os.path.join(module_path, "__init__.py"), "w") as f:
				f.write("")

	def add_to_modules_txt(self):
		"""Adds to `[app]/modules.txt`"""
		modules = None
		if not criscostack.local.module_app.get(criscostack.scrub(self.name)):
			with open(criscostack.get_app_path(self.app_name, "modules.txt")) as f:
				content = f.read()
				if not self.name in content.splitlines():
					modules = list(filter(None, content.splitlines()))
					modules.append(self.name)

			if modules:
				with open(criscostack.get_app_path(self.app_name, "modules.txt"), "w") as f:
					f.write("\n".join(modules))

				criscostack.clear_cache()
				criscostack.setup_module_map()

	def on_trash(self):
		"""Delete module name from modules.txt"""

		if not criscostack.conf.get("developer_mode") or criscostack.flags.in_uninstall or self.custom:
			return

		modules = None
		if criscostack.local.module_app.get(criscostack.scrub(self.name)):
			delete_folder(self.module_name, "Module Def", self.name)
			with open(criscostack.get_app_path(self.app_name, "modules.txt")) as f:
				content = f.read()
				if self.name in content.splitlines():
					modules = list(filter(None, content.splitlines()))
					modules.remove(self.name)

			if modules:
				with open(criscostack.get_app_path(self.app_name, "modules.txt"), "w") as f:
					f.write("\n".join(modules))

				criscostack.clear_cache()
				criscostack.setup_module_map()


@criscostack.whitelist()
def get_installed_apps():
	return json.dumps(criscostack.get_installed_apps())
