# Copyright (c) 2019, Crisco Technologies and Contributors
# License: MIT. See LICENSE
import criscostack
from criscostack.core.doctype.user.user import get_system_users
from criscostack.desk.form.assign_to import add as assign_task
from criscostack.tests.utils import CriscoTestCase


class TestNotificationLog(CriscoTestCase):
	def test_assignment(self):
		todo = get_todo()
		user = get_user()

		assign_task(
			{"assign_to": [user], "doctype": "ToDo", "name": todo.name, "description": todo.description}
		)
		log_type = criscostack.db.get_value(
			"Notification Log", {"document_type": "ToDo", "document_name": todo.name}, "type"
		)
		self.assertEqual(log_type, "Assignment")

	def test_share(self):
		todo = get_todo()
		user = get_user()

		criscostack.share.add("ToDo", todo.name, user, notify=1)
		log_type = criscostack.db.get_value(
			"Notification Log", {"document_type": "ToDo", "document_name": todo.name}, "type"
		)
		self.assertEqual(log_type, "Share")

		email = get_last_email_queue()
		content = "Subject: {} shared a document ToDo".format(
			criscostack.utils.get_fullname(criscostack.session.user)
		)
		self.assertTrue(content in email.message)


def get_last_email_queue():
	res = criscostack.get_all("Email Queue", fields=["message"], order_by="creation desc", limit=1)
	return res[0]


def get_todo():
	if not criscostack.get_all("ToDo"):
		return criscostack.get_doc({"doctype": "ToDo", "description": "Test for Notification"}).insert()

	res = criscostack.get_all("ToDo", limit=1)
	return criscostack.get_cached_doc("ToDo", res[0].name)


def get_user():
	return get_system_users(limit=1)[0]
