// Copyright (c) 2015, Crisco Technologies Pvt. Ltd. and Contributors
// MIT License. See license.txt

criscostack.provide("criscostack.workflow");

criscostack.workflow = {
	state_fields: {},
	workflows: {},
	setup: function (doctype) {
		var wf = criscostack.get_list("Workflow", { document_type: doctype });
		if (wf.length) {
			criscostack.workflow.workflows[doctype] = wf[0];
			criscostack.workflow.state_fields[doctype] = wf[0].workflow_state_field;
		} else {
			criscostack.workflow.state_fields[doctype] = null;
		}
	},
	get_state_fieldname: function (doctype) {
		if (criscostack.workflow.state_fields[doctype] === undefined) {
			criscostack.workflow.setup(doctype);
		}
		return criscostack.workflow.state_fields[doctype];
	},
	get_default_state: function (doctype, docstatus) {
		criscostack.workflow.setup(doctype);
		var value = null;
		$.each(criscostack.workflow.workflows[doctype].states, function (i, workflow_state) {
			if (cint(workflow_state.doc_status) === cint(docstatus)) {
				value = workflow_state.state;
				return false;
			}
		});
		return value;
	},
	get_transitions: function (doc) {
		criscostack.workflow.setup(doc.doctype);
		return criscostack.xcall("criscostack.model.workflow.get_transitions", { doc: doc });
	},
	get_document_state_roles: function (doctype, state) {
		criscostack.workflow.setup(doctype);
		let workflow_states =
			criscostack.get_children(criscostack.workflow.workflows[doctype], "states", { state: state }) ||
			[];
		let allow_edit_list = workflow_states.map((d) => d.allow_edit);
		return allow_edit_list;
	},
	is_self_approval_enabled: function (doctype) {
		return criscostack.workflow.workflows[doctype].allow_self_approval;
	},
	is_read_only: function (doctype, name) {
		var state_fieldname = criscostack.workflow.get_state_fieldname(doctype);
		if (state_fieldname) {
			var doc = locals[doctype][name];
			if (!doc) return false;
			if (doc.__islocal) return false;

			var state =
				doc[state_fieldname] || criscostack.workflow.get_default_state(doctype, doc.docstatus);

			let allow_edit_roles = state
				? criscostack.workflow.get_document_state_roles(doctype, state)
				: null;
			let has_common_role = criscostack.user_roles.some((role) =>
				allow_edit_roles.includes(role)
			);
			return !has_common_role;
		}
		return false;
	},
	get_update_fields: function (doctype) {
		var update_fields = $.unique(
			$.map(criscostack.workflow.workflows[doctype].states || [], function (d) {
				return d.update_field;
			})
		);
		return update_fields;
	},
	get_state(doc) {
		const state_field = this.get_state_fieldname(doc.doctype);
		let state = doc[state_field];
		if (!state) {
			state = this.get_default_state(doc.doctype, doc.docstatus);
		}
		return state;
	},
	get_all_transitions(doctype) {
		return criscostack.workflow.workflows[doctype].transitions || [];
	},
	get_all_transition_actions(doctype) {
		const transitions = this.get_all_transitions(doctype);
		return transitions.map((transition) => {
			return transition.action;
		});
	},
};
